@@ -15,6 +15,7 @@ import (
 	"github.com/sirupsen/logrus"
 
 	"gitlab.com/gitlab-org/gitlab-runner/helpers"
+	"gitlab.com/gitlab-org/gitlab-runner/helpers/featureflags"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/tls"
 	"gitlab.com/gitlab-org/gitlab-runner/session"
 	"gitlab.com/gitlab-org/gitlab-runner/session/terminal"
@@ -68,12 +69,6 @@ const (
 	BuildStageUploadOnFailureArtifacts BuildStage = "upload_artifacts_on_failure"
 )
 
-const (
-	FFDockerHelperImageV2         string = "FF_DOCKER_HELPER_IMAGE_V2"
-	FFUseLegacyGitCleanStrategy   string = "FF_USE_LEGACY_GIT_CLEAN_STRATEGY"
-	FFUseLegacyBuildsDirForDocker string = "FF_USE_LEGACY_BUILDS_DIR_FOR_DOCKER"
-)
-
 type Build struct {
 	JobResponse `yaml:",inline"`
 
@@ -573,11 +568,18 @@ func (b *Build) GetDefaultVariables() JobVariables {
 }
 
 func (b *Build) GetDefaultFeatureFlagsVariables() JobVariables {
-	return JobVariables{
-		{Key: "FF_K8S_USE_ENTRYPOINT_OVER_COMMAND", Value: "true", Public: true, Internal: true, File: false}, // TODO: Remove in 12.0
-		{Key: FFUseLegacyGitCleanStrategy, Value: "false", Public: true, Internal: true, File: false},         // TODO: Remove in 12.0
-		{Key: FFUseLegacyBuildsDirForDocker, Value: "false", Public: true, Internal: true, File: false},       // TODO: Remove in 13.0
+	variables := make(JobVariables, 0)
+	for _, featureFlag := range featureflags.GetAll() {
+		variables = append(variables, JobVariable{
+			Key:      featureFlag.Name,
+			Value:    featureFlag.DefaultValue,
+			Public:   true,
+			Internal: true,
+			File:     false,
+		})
 	}
+
+	return variables
 }
 
 func (b *Build) GetSharedEnvVariable() JobVariable {
@@ -834,17 +836,13 @@ func NewBuild(jobData JobResponse, runnerConfig *RunnerConfig, systemInterrupt c
 }
 
 func (b *Build) IsFeatureFlagOn(name string) bool {
-	ffValue := b.GetAllVariables().Get(name)
+	value := b.GetAllVariables().Get(name)
 
-	if ffValue == "" {
-		return false
-	}
-
-	on, err := strconv.ParseBool(ffValue)
+	on, err := featureflags.IsOn(value)
 	if err != nil {
 		logrus.WithError(err).
-			WithField("ffName", name).
-			WithField("ffValue", ffValue).
+			WithField("name", name).
+			WithField("value", value).
 			Error("Error while parsing the value of feature flag")
 
 		return false
