@@ -10,3 +10,6 @@ const prebuiltImageExtension = ".tar.xz"
 const dockerCleanupTimeout = 5 * time.Minute
 
 const waitForContainerTimeout = 15 * time.Second
+
+const osTypeLinux = "linux"
+const osTypeWindows = "windows"
