@@ -6,6 +6,7 @@ import (
 	"sync"
 
 	"github.com/docker/docker/api/types"
+
 	"gitlab.com/gitlab-org/gitlab-runner/common"
 	"gitlab.com/gitlab-org/gitlab-runner/executors"
 )
@@ -57,7 +58,7 @@ func (s *commandExecutor) requestNewPredefinedContainer() (*types.ContainerJSON,
 		Name: prebuildImage.ID,
 	}
 
-	containerJSON, err := s.createContainer("predefined", buildImage, common.ContainerCommandBuild, []string{prebuildImage.ID})
+	containerJSON, err := s.createContainer("predefined", buildImage, s.helperImageInfo.Cmd, []string{prebuildImage.ID})
 	if err != nil {
 		return nil, err
 	}
@@ -102,6 +103,11 @@ func (s *commandExecutor) Run(cmd common.ExecutorCommand) error {
 }
 
 func init() {
+	initializeLinuxDockerExecutor()
+	initializeWindowsDockerExecutor()
+}
+
+func initializeLinuxDockerExecutor() {
 	options := executors.ExecutorOptions{
 		DefaultCustomBuildsDirEnabled: true,
 		DefaultBuildsDir:              "/builds",
@@ -113,6 +119,9 @@ func init() {
 			RunnerCommand: "/usr/bin/gitlab-runner-helper",
 		},
 		ShowHostname: true,
+		Metadata: map[string]string{
+			"OSType": osTypeLinux,
+		},
 	}
 
 	creator := func() common.Executor {
@@ -141,3 +150,47 @@ func init() {
 		DefaultShellName: options.Shell.Shell,
 	})
 }
+
+func initializeWindowsDockerExecutor() {
+	options := executors.ExecutorOptions{
+		DefaultCustomBuildsDirEnabled: true,
+		DefaultBuildsDir:              `C:\builds`,
+		DefaultCacheDir:               `C:\cache`,
+		SharedBuildsDir:               false,
+		Shell: common.ShellScriptInfo{
+			Shell:         "powershell",
+			Type:          common.NormalShell,
+			RunnerCommand: "gitlab-runner-helper",
+		},
+		ShowHostname: true,
+		Metadata: map[string]string{
+			"OSType": osTypeWindows,
+		},
+	}
+
+	creator := func() common.Executor {
+		e := &commandExecutor{
+			executor: executor{
+				AbstractExecutor: executors.AbstractExecutor{
+					ExecutorOptions: options,
+				},
+			},
+		}
+		e.SetCurrentStage(common.ExecutorStageCreated)
+		return e
+	}
+
+	featuresUpdater := func(features *common.FeaturesInfo) {
+		features.Variables = true
+		features.Image = true
+		features.Services = true
+		features.Session = false
+		features.Terminal = false
+	}
+
+	common.RegisterExecutor("docker-windows", executors.DefaultExecutorProvider{
+		Creator:          creator,
+		FeaturesUpdater:  featuresUpdater,
+		DefaultShellName: options.Shell.Shell,
+	})
+}
